/*
** Module   : GuiWidgetFactory.java
** Abstract : a factory for GUI widgets
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 MAG 20140521 Created initial version.
** 002 CA  20140711 Register the new created window with the GUI driver.
** 003 CA  20140805 Added generics to force the abstract widget classes to not use implementation-
**                  specific code: the abstract widgets will have access only to the base APIs
**                  provided by the OutputManager.
**                  Added create IMAGE widget.
** 004 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 005 MAG 20140917 Added support for scroll bar widget creation.
**                  Added support for scroll pane widget creation.
** 006 MAG 20141009 Deleted createMessageLine. InsertIndicator class has been deleted.
** 007 MAG 20141017 Added GUI frame factory method.
** 008 EVL 20141114 Adding rectangle constructor mapping.  Scroll gui container now creaing here.
** 009 CA  20141123 Removed TODO from createWindow.
** 010 EVL 20141121 Adding button constructor mapping.
** 011 EVL 20141208 Adding image constructor mapping.
** 012 CA  20141220 Added mappings for FILL-IN, LABEL, SKIP, TEXT.
** 013 CA  20150323 Labels are always created using specific APIs; these APIs require the frame to
**                  which the label will belong, initially.
*/

package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.*;
import com.goldencode.p2j.ui.client.driver.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * GUI-specific widget factory implementation.
 * <p>
 * The return type for the {@code create} APIs is different from the {@link WidgetFactory} 
 * versions because:
 * <ul>
 *    <li>it avoids adding at the signature the {@link GuiOutputManager} type, i.e. instead
 *        of using {@code MessageArea&lt;GuiOutputManager&gt;} as return type, 
 *        {@link MessageAreaImpl} can be used directly, with the same effect. </li>
 *    <li> adds an additional protection against incorrect usage of the {@link WidgetFactory} 
 *    implementation classes: if the {@link WidgetFactory} implementation class is used 
 *    explicitly, it prevents accidental mixing of ChUI and GUI widgets (i.e. if used from 
 *    incorrect location). </li>
 * </ul> 
 */
public class GuiWidgetFactory
extends WidgetFactoryAdapter<GuiOutputManager>
{
   static 
   {
      classMap.put(Button.class, ButtonGuiImpl.class);
      classMap.put(FillIn.class, FillInGuiImpl.class);
      classMap.put(Frame.class, FrameGuiImpl.class);
      classMap.put(Image.class, ImageGuiImpl.class);      
      classMap.put(Rect.class, RectangleGuiImpl.class);
      classMap.put(Skip.class, SkipGuiImpl.class);
      classMap.put(Text.class, TextGuiImpl.class);
      classMap.put(Window.class, WindowGuiImpl.class);
      classMap.put(ToggleBox.class, ToggleBoxGuiImpl.class);
   }
   
   /**
    * Create UI-specific {@link Window} instance.
    * 
    * @param   id
    *          The widget's ID.
    * 
    * @return  An instance of the UI-specific implementation of {@link Window}.
    */
   @Override
   public WindowGuiImpl createWindow(WidgetId id)
   {
      WindowGuiImpl window = new WindowGuiImpl(id);
      
      return window;
   }
   
   /**
    * Create UI-specific {@link StatusLine} instance.
    * 
    * @param   id
    *          The widget's ID.
    * 
    * @return  An instance of the UI-specific implementation of {@link StatusLine}.
    */
   @Override
   public StatusLineGuiImpl createStatusLine(WidgetId id)
   {
      return new StatusLineGuiImpl(id);
   }
   
   /**
    * Create UI-specific {@link MessageArea} instance.
    * 
    * @param   id
    *          The widget's ID.
    * @param   lines
    *          Number of visible lines in message area.         
    * 
    * @return  instance of UI-specific implementation of {@link MessageArea}.
    */
   @Override
   public MessageArea<GuiOutputManager> createMessageArea(WidgetId id, int lines)
   {
      return new MessageAreaGuiImpl(id, lines);
   }
   
   /**
    * Create UI-specific {@link Image} instance.
    * 
    * @param   id
    *          The widget's ID.
    * 
    * @return  instance of UI-specific implementation of {@link Image}.
    */
   @Override
   public ImageGuiImpl createImage(WidgetId id)
   {
      return new ImageGuiImpl(id);
   }
   
   /**
    * Create UI-specific {@link ScrollBar} instance.
    * 
    * @param   id
    *          The widget's ID.
    * @param   orientation
    *          Scroll bar orientation horizontal or vertical.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollBarGuiImpl}.
    */
   @Override
   public ScrollBarGuiImpl createScrollBar(WidgetId id, ScrollBar.Orientation orientation)
   {
      return new ScrollBarGuiImpl(id, orientation);
   }

   /**
    * Create UI-specific {@link ScrollContainer} implementation.
    * 
    * @param   isScrollable
    *          The boolean flag that indicates whether the ScrollContainer has
    *          the scrolling capability.
    * @param   frame 
    *          Frame widget which will own the container.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollPane}.
    */
   @Override
   public ScrollGuiContainer createScrollContainer(boolean isScrollable,
                                                   Frame<GuiOutputManager> frame)
   {
      return new ScrollGuiContainer(isScrollable, frame);
   }

   /**
    * Create UI-specific {@link ScrollPane} instance.
    * 
    * @param   id
    *          The widget's ID.
    * @param   widget
    *          Scroll bar orientation horizontal or vertical.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollPaneGuiImpl}.
    */
   @Override
   public ScrollPaneGuiImpl createScrollPane(WidgetId id, Widget<GuiOutputManager> widget)
   {
      return new ScrollPaneGuiImpl(id, widget);
   }
   
   /**
    * Create UI-specific {@link Label} instance as a copy of given {@link Label}.
    * 
    * @param   id
    *          The widget's ID.
    * @param   labelInstance
    *          Source {@link Label} instance.
    * @param   frameId
    *          The ID of the frame to which the label belongs, or -1 if unknown at the time of the 
    *          creation.
    *          
    * @return  instance of UI-specific implementation of {@link Label}.
    */
   @Override
   public LabelGuiImpl createLabel(WidgetId id, Label<GuiOutputManager> labelInstance, int frameId)
   {
      return new LabelGuiImpl(id, labelInstance, frameId);
   }
   
   /**
    * Create new {@link Label} instance for given text.
    * 
    * @param   id
    *          The widget's ID.
    * @param   text
    *          Label text. 
    * @param   frameId
    *          The ID of the frame to which the label belongs, or -1 if unknown at the time of the 
    *          creation.
    * 
    * @return  new {@link Label} instance.
    */
   @Override
   public LabelGuiImpl createLabel(WidgetId id, String text, int frameId)
   {
      return new LabelGuiImpl(id, text, frameId);
   }
}
