/*
** Module   : ToggleBoxGuiImpl.java
** Abstract : GUI toggle-box implementation.
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 VIG 20150210 Initial version.
*/
package com.goldencode.p2j.ui.client.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Stroke;
import java.awt.event.MouseEvent;

import com.goldencode.p2j.ui.FillInConfig;
import com.goldencode.p2j.ui.Keyboard;
import com.goldencode.p2j.ui.ToggleBoxConfig;
import com.goldencode.p2j.ui.WidgetConfig;
import com.goldencode.p2j.ui.WidgetId;
import com.goldencode.p2j.ui.client.NativeDimension;
import com.goldencode.p2j.ui.client.NativePoint;
import com.goldencode.p2j.ui.client.NativeRectangle;
import com.goldencode.p2j.ui.client.OutputManager;
import com.goldencode.p2j.ui.client.SysColor;
import com.goldencode.p2j.ui.client.ToggleBox;
import com.goldencode.p2j.ui.client.Window;
import com.goldencode.p2j.ui.client.event.EventManager;
import com.goldencode.p2j.ui.client.event.EventType;
import com.goldencode.p2j.ui.client.event.Key;
import com.goldencode.p2j.ui.client.event.KeyInput;
import com.goldencode.p2j.ui.client.gui.GuiFontResolver.FontCache;
import com.goldencode.p2j.ui.client.gui.driver.GuiDriver;
import com.goldencode.p2j.ui.client.gui.driver.GuiOutputManager;
import com.goldencode.p2j.ui.client.widget.Container;

/**
 * Specific GUI implementation of TOGGLE-BOX widget.
 */
@SuppressWarnings("unchecked")
public class ToggleBoxGuiImpl
extends ToggleBox<GuiOutputManager>
{
   /** Dotted line specific parameter. */
   private static final float DOT_STEP = 1.0f;
   
   /** Dotted line width. */
   private static final float DOT_LINE_WIDTH = 0.5f;
   
   /** Color resolver */
   private GuiColorResolver gc;

   /** Screen driver */
   private GuiDriver gd;

   /** Font resolver */
   private GuiFontResolver gf;
   
   /** Dotted line pattern for focused button text frame. */
   private Stroke dotStroke;
   
   /** Dark shadow box color. */
   private Color darkShadow;
   
   public ToggleBoxGuiImpl(WidgetId id, Container<GuiOutputManager> owner, String text)
   {
      super(id, owner, text);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public ToggleBoxGuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (ToggleBoxConfig) cfg);
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    wcfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, ToggleBoxConfig wcfg)
   {
      super.initialize(id, wcfg);
      // screen driver
      gd = (GuiDriver) OutputManager.getDriver();

      // font resolver
      gf = new GuiFontResolver(Window.resolveWindow(this), gd, config());

      // initialize dot pattern 
      dotStroke = new BasicStroke(DOT_LINE_WIDTH, BasicStroke.CAP_BUTT,
                                  BasicStroke.JOIN_BEVEL, 0.0f, // this must be just valid
                                  new float[] {DOT_STEP, DOT_STEP}, DOT_STEP);

      // initialize the shadow colors
      darkShadow     = gd.getSysColor(SysColor.COLOR_3DSHADOW);
   }
   /**
    * Draw the toggle-box implementation. 
    */
   @Override
   public void draw()
   {
      final ToggleBoxConfig config = (ToggleBoxConfig) config();
      gc = new GuiColorResolver(gd, config, SysColor.COLOR_MENU, SysColor.COLOR_BTNTEXT);
      
      // determine colors to use
      gc.colors();
      
      final String text = textLabel();
      final FontCache fntCache = gf.font();
      
      // TODO: find out what will be the right coordinates to use here
      final NativePoint p = physicalLocation();
      final NativeDimension d = physicalDimension();

      final NativeRectangle clip = clipRectangle(p, d);
      
      // TODO: clipping does not work properly, need to fix clipRectangle() in AbstractWidget 
      gd.draw(p, null, new Runnable()
      {

         @Override
         public void run()
         {
            // inside this call widget origin is always (1,1)
            // TODO: this is caused by the frame's 1px border... need to fix at the widget 
            // placement, not here. Check other cases too!
            NativePoint po = new NativePoint(1, 1);

            // button state flags
            boolean highlighted = highlighted();
            
            // focused button needs special highlight
            if (highlighted)
            {
               drawHighlight(po, d, text, gc.fgColor, fntCache.font);
            }

            drawCheckBox(po, config);

            int xt = po.x + 6 + (d.width - gd.getTextWidth(text, fntCache.font))/2 + 6;
            int yt = po.y - 3 + (d.height + fntCache.font.getSize())/2;
            
            gd.setColor(gc.fgColor);
            gd.drawString(text, xt, yt);
         }

         
      });
   }

   /**
    * Draw highlight frame indicating focused button.
    * 
    * @param   orig
    *          Rectangle origin.
    * @param   dim
    *          Rectangle size.
    * @param   color
    *          Border color.
    */
   private void drawHighlight(NativePoint orig, NativeDimension dim, String label, Color color, Font font)
   {
      gd.setColor(color);
   
      // set dotted line style      
      gd.setLineStroke(dotStroke);
      
      int textWidth = gd.getTextWidth(label, font);
      int xt = orig.x - 1 + 6 + (dim.width - textWidth)/2 + 6;
      int yt = orig.y; 
      
      gd.drawRect(xt, yt, textWidth, dim.height - 4);
      
      // restore default line style
      gd.setLineStroke(new BasicStroke());
   }

   /**
    * Draws check box 
    * 
    * @param po
    *        origin point.
    * @param config
    *        toggle-box config       
    */
   private void drawCheckBox(NativePoint po, ToggleBoxConfig config)
   {
      // left top angle
      gd.setColor(gc.fgColor);
      gd.drawLine(po.x + 6, po.y + 3, po.x + 17, po.y + 3);
      gd.drawLine(po.x + 6, po.y + 4, po.x + 6, po.y + 14);
      gd.drawLine(po.x + 7, po.y + 4, po.x + 16, po.y + 4);
      gd.drawLine(po.x + 7, po.y + 4, po.x + 7, po.y + 13);
      
      // right bottom angle
      gd.setColor(darkShadow);
      gd.drawLine(po.x + 8, po.y + 14, po.x + 17, po.y + 14);
      gd.drawLine(po.x + 17, po.y + 5, po.x + 17, po.y + 14);
      
      // draw check
      if (config.checked)
      {
         gd.setColor(gc.fgColor);
         // left half
         gd.drawLine(po.x + 9, po.y + 8, po.x + 11, po.y + 10);
         gd.drawLine(po.x + 9, po.y + 9, po.x + 11, po.y + 11);
         gd.drawLine(po.x + 9, po.y + 10, po.x + 11, po.y + 12);
         // right half
         gd.drawLine(po.x + 15, po.y + 6, po.x + 11, po.y + 10);
         gd.drawLine(po.x + 15, po.y + 7, po.x + 11, po.y + 11);      
         gd.drawLine(po.x + 15, po.y + 8, po.x + 11, po.y + 12);      
      }
   }
   
   /**
    * Process GUI specific keys:
    * 
    * @param   ke
    *          Key event structure.
    */
   @Override
   public void processKeyEvent(KeyInput ke)
   {
      int key = ke.actionCode();
      
      if (key == Key.VK_SPACE)
      {
         config().checked = !config().checked;
         
         repaint();
         
         ke.consume();
      }
      
      super.processKeyEvent(ke);
   }

   /**
    * Notification of a mouse clicked event occurred for this widget.  Simulate ENTER key typing.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseClicked(MouseEvent e)
   {
      // simulate enter key
      if (isEnabled())
      {
         EventManager.postEvent(new KeyInput(this, EventType.KEY_TYPED, Key.VK_SPACE));
      }
   }
}
